%
% Generates fractional ARIMA time series with stable symmetric innovations
% by using a truncated version of the moving average representation developed 
% by Kokoszka & Taqqu, Stoch. Processes and Their Appl. 60, 19-47, 1995. 
%
% input:
%   alpha <- the tail-exponent of the innovations (must be > 1)
%   d     <- the fractional differencing exponent (must be >0, <1-1/alpha)  
%   n     <- a number of independent paths to be generated
%   Ph    <- a vector of the AR (autoregressive) coefficients
%            [] means "no AR part"
%   Th    <- a vector of the MA (moving average) coefficients 
%            [] means "no MA part"
%   M     <- the cut-off parameter
%   N     <- the desired length of the moving average
%
% output:
%   Y     <- a (nxN) matrix each row corresponds to a generated path
%            of FARIMA(p,d,q). The paths are "independent".
% usage:
%        Y = fftfarima(alpha,beta,d,n,Ph,Th,M,N,outp);
% 
% ***************************************************************************
% *NOTEs: 1. When M+N is a  powers of 2, the FFT algorithm is used.
% *       2. Calls: fracdiff.m 
% *       3. Phi(z)  = 1 - phi(1)z - phi(2)z^2 - ... - phi(p)z^p
% *          Theta(z)= 1 - theta(1)z- theta(2)z^2 - ... - theta(q)z^q,
% *          where Ph = [phi(1) .. phi(p)]; Th = [theta(1) .. theta(q)]
% *       4. Suggested values M=N=2^20, n=1000;
% ***************************************************************************
%  Written by Stilian Stoev, 05.05.2002, sstoev@math.bu.edu.
%
function Y = fftfarima(alpha,beta,d,n,Ph,Th,M,N,outp);
b = fracdiff(d,M);
c = filter([1, -Th],[1, -Ph],b);
c = [c, zeros(1,N)];
C = fft(c,M+N);
Y = [];
if outp,
    fprintf(' Generating n = %d paths of FARIMA',n);
end;
for i=1:n,
 
 if alpha<2,
   Z = rstab(alpha,beta,M+N)'; 
 elseif alpha==2
   Z = randn(1,M+N);
 end;
 Zhat = fft(Z,M+N);
 y    = real(ifft(C.*Zhat,M+N));
 Y    = [Y; y(1:N)];  
   if outp & (mod(i,fix(n/10))==0) ,
       fprintf('.');
   end;
end;
if outp,
    fprintf(' done!');
end;


