%
% This is the main function of the MATLAB package used to generate
% alpha-stable variates.  It includes the MATLAB scripts 
% rstab.m (this function), tan2.m and d2.m, which were adapted 
% by Dimitar Vandev <vandev@fmi.uni-sofia.bg> from the original FORTAN code
% in the paper of Chambers, Mallows and Stuck, JASA (1976, correction
% in 1986).
%
% INPUT:
%  alpha <- the index of stability (0<alpha<2)
%  brime <- the beta or the skweness parameter
%  n     <- the size of the sample
%
% OUTPUT:
%  y     <- a vector of the simulated (alpha,beta)-stable variates
% 
% USAGE:
%  y = rstab(alpha,brime,n);
%
% *************************************************************************
% * IMPORTANT:
% The correction in Chambers, Mallows and Stuck JASA (1987) was
% incorporated in the function "d2.m".
% *************************************************************************
%
function y=rstab(alpha,brime,n)
%     function rstab(alpha,brime,n)
%
%  GENERATE A STABLE PARETIAN VARIATE-P.BRATLEY,B.FOX,L.SCHRAGE-560 B9R
%
%  INPUTS
%  ALPHA=CHARACTERISTIC EXPONENT,0<=ALPHA<=2
%  BRIME=SKEWNESS IN REVISED PARAMETRIZATION,SEE REF.
%	=0 GIVES A SYMMETRIC DISTRIBUTION.
%  n = number of variates
%
%  OUTPUT: y(n,1)
%  REFERENCE: CHAMBERS,J.M.,C.L.MALLOWS,B.W.STUCK(1976)
%  "A METHOD FOR SIMULATING STABLE RANDOM VARIABLES",JASA,VOL.71,
%  NO. 354, PP. 340-344
%
%  AUXILIARY ROUTINES: D2,TAN2
      pi2=pi/2;
      thr1=0.99;
      eps=1.-alpha;
      if (eps >-.99)
	   tau=brime/(pi2*tan2(eps*pi2));
	 else
	   tau=pi2*alpha;
	   tau=brime*eps*tau*tan2(tau);
      end
%
%  COMPUTE SOME TANGENTS
%
      a=rand(n,1);
      phiby2=(a-.5)*pi2;
      a=phiby2.*tan2(phiby2);
      b=phiby2*eps;
      bb=tan2(b);
      b=b.*tan2(b);
%
%  COMPUTE SOME NECESSARY SUBEXPRESSIONS
%  IF PHI NEAR PI BY 2, USE DOUBLE PRECISION.
%
      da=a.*a;
      a2=1-da;
      a2p=1+da;da=[];
      db=b.*b;
      b2=1-db;
      b2p=1+db;db=[];
%
%  COMPUTE COEFFICIENT
%
      z=-log(rand(n,1));
      z=z.*a2.*b2p;
      z=log(a2p.*(b2+phiby2.*bb*(2*tau))./z);
      z=z/(1-eps);
      d=z.*d2(z*eps);
%
%  COMPUTE STABLE
%
      rstab=d*eps; y=a.*b;y=1+y;
      rstab=(1+d*eps);
      rstab=(rstab*2).*...
      ((a-b).*y-(phiby2*tau).*bb.*(b.*a2-a*2))./...
      (a2.*b2p)+d*tau;
      y=rstab;
%
%
% Adapted from the original FORTRAN code by Dimitar Vandev 
% (vandev@fmi.uni-sofia.bg)
%
%





